package edu.uiuc.cs.net.DPRPManager;
//package DPRPManager;

import java.io.*;
import java.util.*;
import java.net.*;

/**
 * This class represents the command-line front-end to the DPRP
 * Client. It parses a bunch of command-line parameters, and then
 * kicks off a new <b><tt>DPRPClientSession</tt></b> thread based upon those
 * parameters. Once a lease has been obtained, it spits out the
 * parameters in a semi-human-readable format. Further, it will
 * continue to run, renewing the lease as necessary.
 *
 * @version 1.5 - October 16th, 2001
 * @author Andy Reitz (<a href="mailto:areitz@cs.uiuc.edu">areitz@cs.uiuc.edu</a>)
 */
public class CLI_DPRPClient implements DPRPClientConstants, DPRPLeaseListenerConstants {
    /**
     * Starts the command-line client.
     * @param args an array of command-line arguments
     */
    public static void main (String args[])
    {
	Thread ourThread = Thread.currentThread();
	DPRPClient ourClient = new DPRPClient (ourThread);

	/*
	 * The 'new' way to gather command-line parameters.
	 * This is adapted from the "Code Red Vigilante":
	 *   http://www.dynwebdev.com/codered/
	 * Specifically, the file 'Bait.java'.
	 */
	try {
	    Integer tmp_i = null;
	    Short tmp_s = null;
	    int i = 0;
	    for (i = 0; i < args.length; i++) {
		if (args[i].equals ("--srvPort") || args[i].equals ("-s")) {
		    tmp_i = new Integer (args[++i]);
		    //srvPort = tmp_i.intValue();
		    ourClient.setServerPort (tmp_i.intValue());
		    // slap it right into the client here
		} else if (args[i].equals ("--reqDuration") ||
			   args[i].equals ("-u")) {
		    tmp_i = new Integer (args[++i]);
		    //reqDuration = tmp_i.intValue();
		    ourClient.setDuration (tmp_i.intValue());
		} else if (args[i].equals ("--reqPort") ||
			   args[i].equals ("-r")) {
		    tmp_i = new Integer (args[++i]);
		    //reqPort = tmp_i.intValue();
		    ourClient.setRequestedPort (tmp_i.intValue());
		} else if (args[i].equals ("--flags") ||
			   args[i].equals ("-f")) {
		    tmp_s = new Short (args[++i]);
		    //reqFlags = tmp_s.shortValue();
		    ourClient.setFlags (tmp_s.shortValue());
		} else if (args[i].equals ("--debug") ||
			   args[i].equals ("-d")) {
		    DPRPClient.DEBUG = true;
		    DPRPLease.DEBUG = true;
		} else if (args[i].equals ("--protocol") ||
			   args[i].equals ("-p")) {
		    if (args[i + 1].equalsIgnoreCase ("tcp")) {
			//reqProtocol = DPRPMessage.TCP_PROTOCOL;
			ourClient.setProtocol (DPRPMessage.TCP_PROTOCOL);
			i++;
		    } else if (args[i + 1].equalsIgnoreCase ("udp")) {
			//reqProtocol = DPRPMessage.UDP_PROTOCOL;
			ourClient.setProtocol (DPRPMessage.UDP_PROTOCOL);
			i++;
		    } else
			throw new Exception ("Error: invalid protocol " +
					     "specification");
		} else if (args[i].equals ("--intAddr") ||
			   args[i].equals ("-i")) {
		    //cliIP = strToInetAddr (args[++i]);
		    ourClient.setClientIP (InetAddress.getByName (args[++i]));
		} else if (i == args.length - 1) {
		    //srvIP = InetAddress.getByName (args[i]);
		    ourClient.setServerIP (InetAddress.getByName (args[i]));
		} else
		    throw new Exception ("Error: Server wasn't specified.");

	    } /* for */
	} catch (UnknownHostException e) {
	    System.err.println ("Invalid IP address specified.");
	    printCommandUsage();
	} catch (NumberFormatException e) {
	    System.err.println ("Invalid port/duration specified. It " +
				"must be an integer number, from 1 " +
				"to 65536.");
	    printCommandUsage();
	} catch (Exception e) {
	    System.err.println (e);
	    printCommandUsage();
	}

	// Enforce required parameters. Probably should require protocol.
	if (ourClient.getServerIP() == null) {
	    System.err.println ("Error: server not specified. This " +
				"parameter is required.");
	    printCommandUsage();
	}

	if (DPRPClient.DEBUG)
	    ourClient.dumpParameters();

	/*****
	 * EVERYTHING from here down, needs to be refactored into a new
	 * object -- 'DPRPClientThread', or something. This CLI code
	 * will just spawn a new thread, that does all of this stuff,
	 * and simply do a wait() (or join()) -- exiting when this
	 * new thread does.
	 *****/

	// Now, create a new client object, and set it a rockin'.
	ourClient.kickoffNewRequest();

	// This is all-better now. Essentially, we keep sleeping for a
	// second, and if we're interrupted during that time, we process
	// the variables that have been set in our client object, to
	// see what we're being notified about. It can handle all sorts
	// of errors/notifications, so that's all good. But, this is
	// essentially polling. What I really want to do, is sleep for
	// a long time, and if nothing happens, error-out. I think that
	// there is a method for that...
	DPRPLeaseEvent ev = null;
	boolean gotError = false;
	while (!gotError) {
	    try {
		// This pretty-much amounts to polling. Gak. Instead of
		// 'gotError', should I just break?
		Thread.sleep (1000);
	    } catch (InterruptedException e) {
		ev = ourClient.returnLeaseEvent;
		if (ev.getCommand() == NEW_LEASE_EVENT) {
		    // This doesn't mean that we got a lease, eh bucko?
		    if (ev.isError()) {
			// actually, we got an error.
			System.out.println ("New lease was not acquired. " +
                                  "Reason:\n\t" +
                                  DPRPErrors.resolveError (ev.getErrorcode()));
			gotError = true;
		    } else {
			// Fantastic. Print it, baby.
			System.out.println ("New lease was acquired:");
			ev.getLease().prettyPrint();
		    }
		} else if (ev.getCommand() == UPDATE_LEASE_EVENT) {
		    System.out.println ("Current lease successfully updated:");
		    ev.getLease().prettyPrint();
// 		} else if (ev.getCommand() == CLI_TERMINATE_NORM) {
// 		    // can this ever happen with this client?
// 		    System.out.println ("Lease terminated normally.");
// 		    gotError = true;
		} else if (ev.getCommand() == REMOVE_LEASE_EVENT) {
		    // Oh great, an error by assumption. Deal with it.
		    if (ev.getErrorcode() == DPRPErrors.CLI_SERVER_NACK_ERROR)
			System.out.println ("The server explicitly denied " +
					    "our request. Reason:\n\t" +
					    DPRPErrors.resolveError (ev.getLease().getNackError()));
		    else 
			System.out.println ("Error occured during lease " +
					    "operation. Error message:\n\t" +
					    DPRPErrors.resolveError (ev.getErrorcode()));
		    gotError = true;
		} else {
		    // I don't know what in the hell this is.
		    System.out.println ("Unknown event command (" +
					ev.getCommand() +
					") received from client session.");
		    gotError = true;
		}
	    } /* catch. */
	} /* while. */

	// Force-quit.
	System.exit (0);
    } /* End main(). */

    /**
     * Prints the command usage to stderr, so that users can know how it's
     * done.
     */
    private static void printCommandUsage()
    {
	System.err.println ("Invalid command-line parameters specified." +
			    "\nCorrect usage: " + 
			    "DPRPClient [OPTIONS] <Server Address>");
	System.err.println ("\nOptions:");
	System.err.println ("  -s,  --srvPort\tPort to connect to on server.");
	System.err.println ("  -u,  --reqDuration\tSuggested lease duration.");
	System.err.println ("  -r,  --reqPort\tRequested external port.");
	System.err.println ("  -f,  --flags\t\tFlags to send to server.");
	System.err.println ("  -p,  --protocol\tProtocol of port, either " +
			    "TCP or UDP.");
	System.err.println ("  -d,  --debug\t\tDisplay debugging messages.");
	System.err.println ("  -i,  --intAddr\tThe local IP address to " +
			    "use for outgoing connections.");
	System.err.println ("\t\t\t(optional, only for machines with " +
			    "multiple interfaces)");
	System.exit(1);
    } /* End printCommandUsage(). */

} /* End class CLI_DPRPClient. */
