package edu.uiuc.cs.net.DPRPManager;
//package DPRPManager;

/**
 * Represents a <i>"Lease Event"</i>, which can either be lease creation,
 * updation (?), or deletion. Additionally, for each type of lease event,
 * there is a certain sub-type -- wheather or not this event represents
 * a success or a failure.
 *
 * @version 1.1 - October 29th, 2001
 * @author Andy Reitz (<a href="mailto:areitz@cs.uiuc.edu">areitz@cs.uiuc.edu</a>)
 */

public class DPRPLeaseEvent extends java.util.EventObject {
    /**
     * The <code>DPRPLease</code> object that we are sending.
     */
    protected DPRPLease sentLease;
    /**
     * A letter-code giving a reason why this event was fired. This
     * is really for the JTable's benefit, mostly.
     */
    protected char leaseCommand;
    /** True if this event represents an error of some kind. */
    protected boolean isError;
    /** True if this lease is genuinely new, false otherwise. */
    protected boolean isNew;
    /** The details about the error; only set when isError is true. */
    protected short errorcode;

    /**
     * Creates a new <b><tt>DPRPLeaseEvent</tt></b> object, for notifying
     * the controller of a "lease change event".
     * @param so A reference to the source of this event.
     * @param lea A reference to the lease to be sent in the event.
     * @param reason A control code, designating the type of event.
     */
    public DPRPLeaseEvent (Object so, DPRPLease lea, char reason) {
	super (so);
	sentLease = lea;
	leaseCommand = reason;
	isError = false;
	isNew = true;
	errorcode = 0;
    }

    /**
     * Highly experimental and inflammatory.
     */
    public void setSource (Object sou) {
	source = sou;
    }
    /** Returns the lease that is attached to this event. */
    public DPRPLease getLease() {
	return (sentLease);
    }
    /** Attaches a lease to this event. */
    public void setLease (DPRPLease l) {
	sentLease = l;
    }
    /** Returns the reason that this event was fired. */
    public char getCommand() {
	return (leaseCommand);
    }
    /** Sets the reason for firing this event. */
    public void setCommand (char co) {
	leaseCommand = co;
    }
    /** Marks this event as an error event. */
    public void markAsError() {
	isError = true;
    }
    /** Checks to see if this is an error event. */
    public boolean isError() {
	return (isError);
    }
    /** Gets the errorcode contained in this event. */
    public short getErrorcode() {
	return (errorcode);
    }
    /** Gives this event an errorcode to transmit. */
    public void setErrorcode (short e) {
	errorcode = e;
    }
    /** Checks to see if this is an event for a newly-created lease. */
    public boolean isNew() {
	return (isNew);
    }
    /** Sets this lease to appear "not so new", as such. */
    public void markAsOld() {
	isNew = false;
    }

    /**
     * Stupid debugging method, used in-conjunction with this
     * <b><tt>DPRPLeaseListener</tt></b> stuffs.
     */
    public static String resolveLeaseCommand (char cmd) {
	if (cmd == 'r')
	    return ("REMOVE_LEASE_EVENT");
	else if (cmd == 'n')
	    return ("NEW_LEASE_EVENT");
	else if (cmd == 'u')
	    return ("UPDATE_LEASE_EVENT");
	else
	    return ("UNKNOWN_COMMAND");
    } /* End resolveLeaseCommand(). */

} /* End class DPRPLeaseEvent. */

